package com.temboo.facebook.oauth;

import java.io.IOException;
import java.security.SecureRandom;

// Temboo imports
import com.temboo.Library.Facebook.Reading.User;
import com.temboo.Library.Facebook.Reading.User.UserInputSet;
import com.temboo.Library.Facebook.Reading.User.UserResultSet;
import com.temboo.Library.Facebook.OAuth.FinalizeOAuth;
import com.temboo.Library.Facebook.OAuth.FinalizeOAuth.FinalizeOAuthInputSet;
import com.temboo.Library.Facebook.OAuth.FinalizeOAuth.FinalizeOAuthResultSet;
import com.temboo.Library.Facebook.OAuth.InitializeOAuth;
import com.temboo.Library.Facebook.OAuth.InitializeOAuth.InitializeOAuthInputSet;
import com.temboo.Library.Facebook.OAuth.InitializeOAuth.InitializeOAuthResultSet;
import com.temboo.core.TembooSession;


// A helper class for Facebook OAuth.
public final class FacebookAuthHelper {

	// Provide your Facebook App ID and App Secret.
	private static final String APP_ID = "YOUR_FACEBOOK_APP_ID";
	private static final String APP_SECRET = "YOUR_FACEBOOK_APP_SECRET";

	// Callback URI that Temboo will redirect to after successful authentication.
	// Note that a random state parameter gets appended to this in the Initialize step.
	private static final String FORWARDING_URL = "http://localhost:8080/FacebookOAuthv1/index.jsp";
	
	private String stateToken;
	
	// Replace with your Temboo credentials.
    private static final String TEMBOO_ACCOUNT_NAME = "ACCOUNT_NAME";
    private static final String TEMBOO_APP_KEY_NAME = "APP_KEY_NAME";
    private static final String TEMBOO_APP_KEY_VALUE = "APP_KEY_VALUE";

    private TembooSession session = null;
	
	/**
	 * Constructor initializes the Facebook Authorization Flow.
	 */
	public FacebookAuthHelper() {
		
		generateStateToken();

            // Instantiated TembooSession object.
            try {
                session = new TembooSession(TEMBOO_ACCOUNT_NAME, TEMBOO_APP_KEY_NAME, TEMBOO_APP_KEY_VALUE);
            } catch (Exception te) {
                te.printStackTrace();
            }
	}

	/**
	 * Executes Facebook.OAuth.InitializeOAuth in order to get an AuthorizationURL
	 */
	public String getLoginUrl() {

            String authURL = "";
            try {
            	// Instantiate the InitializeOAuth choreo, using a session object.
                InitializeOAuth initializeOauthChoreo = new InitializeOAuth(session);

                // Get an input set for InitializeOAuth.
                InitializeOAuthInputSet initializeOauthInputs = initializeOauthChoreo.newInputSet();
           
           		// Set inputs for InitializeOAuth.
           		// Use a state token as the custom callback id and in a state parameter of the ForwardingURL.
                initializeOauthInputs.set_AppID(APP_ID);
                initializeOauthInputs.set_CustomCallbackID(stateToken);
                initializeOauthInputs.set_ForwardingURL(FORWARDING_URL + "?state=" + stateToken);

                // Execute InitializeOAuth choreo.
                InitializeOAuthResultSet initializeOauthResults = initializeOauthChoreo.execute(initializeOauthInputs);

                // This is the URL that the user will be directed to in order to login to FB and allow access.
                authURL = initializeOauthResults.get_AuthorizationURL();
                System.out.println("~~~AUTHORIZATION URL: " + authURL);
            } catch (Exception te) {
                te.printStackTrace();
            }
            return authURL;
	}
	
	/**
	 * Generates a secure custom callback ID. This is used as the CustomCallbackID and sent 
	 * as the state parameter in the ForwardingURL when the user is redirected back
	 * to http://localhost:8080/FacebookOAuthv1/index.jsp.
	 */
	private void generateStateToken(){
		
		SecureRandom random = new SecureRandom();
		
		stateToken = "facebook-"+random.nextInt();
		
	}
	
	/**
	 * Accessor for state token.
	 */
	public String getStateToken(){
		return stateToken;
	}
	
	/**
	 * Expects a state parameter which was passed as the CustomCallbackID and in the ForwardingURL
	 * @return JSON response from Facebook.Reading.User choreo
	 */
	public String getUserInfo(final String state) throws IOException {

            	String userInfo = "";
                try {
       				// Instantiate the FinalizeOAuth choreo, using a session object.
                    FinalizeOAuth finalizeOauthChoreo = new FinalizeOAuth(session);
                    FinalizeOAuthInputSet finalizeOauthInputs = finalizeOauthChoreo.newInputSet();
               
               		// Set input for FinalizeOAuth choreo.
                    finalizeOauthInputs.set_AppID(APP_ID);
                    finalizeOauthInputs.set_AppSecret(APP_SECRET);
                    finalizeOauthInputs.set_LongLivedToken("1");

                    final String customCallbackID = TEMBOO_ACCOUNT_NAME + "/" + state;
                    finalizeOauthInputs.set_CallbackID(customCallbackID);

                    // Execute FinalizeOAuth choreo.
                    System.out.println("~~~EXECUTING FINALIZE WITH CLIENT CALLBACK ID: " + customCallbackID);
                    FinalizeOAuthResultSet finalizeOauthResults = finalizeOauthChoreo.execute(finalizeOauthInputs);
                    System.out.println("~~~USER ACCESS TOKEN: " + finalizeOauthResults.get_AccessToken());

                    // Get user info. Instantiate the choreo, using a session object.
                	User userChoreo = new User(session);

					// Get an InputSet object for the Facebook.Reading.User choreo.
					UserInputSet userInputs = userChoreo.newInputSet();

					// Pass access token to the Facebook.Reading.User choreo.
					userInputs.set_AccessToken(finalizeOauthResults.get_AccessToken());

					// Execute Facebook.Reading.User choreo.
					UserResultSet userResults = userChoreo.execute(userInputs);
					userInfo = userResults.get_Response();
                } catch (Exception te) {
                    te.printStackTrace();
                }
                return userInfo;
                // At this point you may want to put the user info in a store or database.
	}
}
